package auth.interceptor;

import com.bwie.auth.utils.ApiIdempotent;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.messaging.handler.HandlerMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author : ZhangBingYan
 * @className : ApiIdempotentInterceptor
 * @description : 描述说明该类的功能
 * @createTime : 2023/10/4 17:33
 * @updateUser : ZhangBingYan
 * @updateTime : 2023/10/4 17:33
 * @updateRemark : 描述说明本次修改内容
 */
// 拦截器
@Component
public class ApiIdempotentInterceptor implements HandlerInterceptor {
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (handler instanceof HandlerMethod) {
            HandlerMethod handlerMethod = (HandlerMethod) handler;
            ApiIdempotent apiIdempotent = handlerMethod.getMethodAnnotation(ApiIdempotent.class);
            if (apiIdempotent != null) {
                String token = request.getHeader("token"); // 或者从请求参数中获取
                if (StringUtils.isEmpty(token)) {
                    response.getWriter().print("token is null");
                    return false;
                }
                // 判断 token 是否存在于缓存中
                if (!redisTemplate.hasKey(token)) {
                    response.getWriter().print("重复的请求");
                    return false;
                }
                // 标记 token 已使用
                redisTemplate.delete(token);
            }
        }
        return true;
    }
}
