package auth.interceptor;

import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author : ZhangBingYan
 * @className : RepeatSubmitInterceptor
 * @description : 描述说明该类的功能
 * @createTime : 2023/10/4 18:54
 * @updateUser : ZhangBingYan
 * @updateTime : 2023/10/4 18:54
 * @updateRemark : 描述说明本次修改内容
 */
@Component
public class RepeatSubmitInterceptor implements HandlerInterceptor {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String key = request.getRequestURI() + "?" + request.getQueryString();
        if (request.getMethod().equalsIgnoreCase("POST")) {
            // POST 请求
            if (isRepeatSubmit(request, key)) {
                // 存在相同的请求，说明是重复提交
                response.sendError(HttpStatus.BAD_REQUEST.value(), "请勿重复提交请求");
                return false;
            } else {
                // 该请求尚未处理，将其存入缓存中
                cacheRequest(request, key);
                return true;
            }
        } else {
            // 其他类型的请求直接放行
            return true;
        }
    }

    // 判断请求是否重复
    private boolean isRepeatSubmit(HttpServletRequest request, String key) {
        // 从缓存中读取上一次相同请求的时间戳
        Object lastTimestamp = request.getSession().getAttribute(key);
        long currentTimestamp = System.currentTimeMillis();
        if (lastTimestamp == null) {
            // 第一次请求，记录时间戳，并返回不重复
            request.getSession().setAttribute(key, currentTimestamp);
            return false;
        } else {
            // 上一次请求的时间戳与当前时间戳差值小于 3 秒，则视为重复请求
            long last = Long.parseLong(lastTimestamp.toString());
            if (currentTimestamp - last < 3000) {
                return true;
            } else {
                request.getSession().setAttribute(key, currentTimestamp);
                return false;
            }
        }
    }

    private void cacheRequest(HttpServletRequest request, String key) {
        // 将请求存储到 session 或者其他缓存中
        request.getSession().setAttribute(key, System.currentTimeMillis());
    }
}
