package auth.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.feign.SysFeign;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.SysUser;
import com.bwie.common.domain.request.UserRequest;
import com.bwie.common.domain.response.UserResponse;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @author : ZhangBingYan
 * @className : AuthServiceImpl
 * @description : 描述说明该类的功能
 * @createTime : 2023/7/28 9:33
 * @updateUser : ZhangBingYan
 * @updateTime : 2023/7/28 9:33
 * @updateRemark : 描述说明本次修改内容
 */
@Service
public class AuthServiceImpl implements AuthService {
    @Autowired
    private SysFeign sysFeign;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    @Autowired
    private HttpServletRequest request;
    /**
     * 账号登录
     * */
    @Override
    public Result<UserResponse> login(UserRequest userRequest) {
        if (StringUtils.isAnyBlank(userRequest.getUserName(),userRequest.getUserPwd())){
            return Result.error("账号或者密码不正确");
        }
        if (StringUtils.isEmpty(userRequest.getUserName())){
            return Result.error("账号错误");
        }
        if (StringUtils.isEmpty(userRequest.getUserPwd())){
            return Result.error("密码错误");
        }
        //远程调用
        Result<SysUser> byName = sysFeign.findByName(userRequest.getUserName());
        SysUser user = byName.getData();
        if (user==null){
            return Result.error("数据不匹配");
        }
        //创建userKey
        String userKey = UUID.randomUUID().toString().replaceAll("-", "");
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY,userKey);
        //创建token
        String token = JwtUtils.createToken(map);
        //存入redis
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+userKey, JSONObject.toJSONString(user),1, TimeUnit.HOURS);
        //序列化
        UserResponse userResponse = new UserResponse();
        userResponse.setToken(token);
        return Result.success(userResponse);
    }
    /**
     * 响应消息
     * */
    @Override
    public SysUser info() {
        //获取token
        String token = request.getHeader(TokenConstants.TOKEN);
        if (StringUtils.isEmpty(token)){
            return null;
        }else {
            //解析token
            String userKey = JwtUtils.getUserKey(token);
            //获取userKey
            String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
            //反序列化
            return JSONObject.parseObject(s, SysUser.class);
        }
    }


    @Override
    public void logout() {
        //获取token
        String token = request.getHeader(TokenConstants.TOKEN);
        //解析token
        String userKey = JwtUtils.getUserKey(token);
        //删除userKey
        redisTemplate.delete(TokenConstants.LOGIN_TOKEN_KEY+userKey);
    }
}
